<?php namespace Phpcmf\Controllers\Admin;

class Home extends \Phpcmf\Common
{
    public function __construct(...$params)
    {
        parent::__construct(...$params);
        \Phpcmf\Service::V()->assign([
            'menu' => \Phpcmf\Service::M('auth')->_admin_menu(
                [
                    '百度熊掌号设置' => ['bdxz/home/index', 'fa fa-paw'],
                    '推送日志' => ['bdxz/home/log_index', 'fa fa-calendar'],
                    'help' => ['673'],
                ]
            ),
        ]);
    }

    public function index() {

        if (IS_AJAX_POST) {


            $post = \Phpcmf\Service::L('Input')->post('data', true);

            if ($post['bdxz']) {
                $bdxz = [];
                foreach ($post['bdxz'] as $i => $t) {
                    if (isset($t['site'])) {
                        if (!$t['site']) {
                            $this->_json(0, dr_lang('域名必须填写'));
                        }
                        $bdxz[$i]['site'] = $t['site'];
                    } elseif (isset($t['appid'])) {
                        if (!$t['appid']) {
                            $this->_json(0, dr_lang('appid必须填写'));
                        }
                        $bdxz[$i-1]['appid'] = $t['appid'];
                    } else {
                        if (!$t['token']) {
                            $this->_json(0, dr_lang('token必须填写'));
                        }
                        $bdxz[$i-2]['token'] = $t['token'];
                    }
                }
                $post['bdxz'] = $bdxz;
            }
            \Phpcmf\Service::M('bdxz', 'bdxz')->setConfig($post);
            \Phpcmf\Service::L('Input')->system_log('设置百度熊掌号工具');
            exit($this->_json(1, dr_lang('操作成功')));
        }

        $page = intval(\Phpcmf\Service::L('Input')->get('page'));

        \Phpcmf\Service::V()->assign([
            'page' => $page,
            'data' => \Phpcmf\Service::M('bdxz', 'bdxz')->getConfig(),
            'form' => dr_form_hidden(['page' => $page]),
            'module' => \Phpcmf\Service::M('Module')->All(1),
        ]);
        \Phpcmf\Service::V()->display('config.html');
    }

    // 历史提交
    public function bdxz_index() {

        $mid = dr_safe_filename($_GET['mid']);
        $page = max(1, intval($_GET['page']));

        $total = \Phpcmf\Service::M()->table(SITE_ID.'_'.$mid)->where('status', 9)->counts();
        if (!$total) {
            $this->_json(1, dr_lang('模块没有任何内容'), 0);
        }

        $psize = 1000;
        $tpage = ceil($total/$psize);

        if ($page > $tpage) {
            $this->_json(1, dr_lang('已提交'), 0);
        }

        $pc_urls = $mb_urls = [];
        $pc_site = $mb_site = '';

        $result = \Phpcmf\Service::M()->db->table(SITE_ID.'_'.$mid)->orderBy('id ASC')
            ->limit($psize, $psize * ($page - 1))->get()->getResultArray();
        if ($result) {
            foreach ($result as $t) {
                $url = dr_url_prefix($t['url'], $mid, SITE_ID, 0); // pc
                if ($url) {
                    $uri = parse_url($url);
                    $pc_site = $uri['host'];
                    if ($pc_site) {
                        $pc_urls[] = $url;
                        $url = dr_url_prefix($t['url'], $mid, SITE_ID, 1); // mobile
                        $uri = parse_url($url);
                        if ($uri['host'] != $pc_site) {
                            // 存在移动端域名时
                            $mb_site = $uri['host'];
                            $mb_urls[] = $url;
                        }

                    }
                }
            }
        }


        if ($mb_urls) {
            // 当存在移动端url就不用pc
            $pc_urls = [];
            $pc_site = '';
        }

        $config = \Phpcmf\Service::M('bdxz', 'bdxz')->getConfig();
        if ($config['bdxz']) {
            $pc_appid = '';
            $pc_token = '';
            $mb_appid = '';
            $mb_token = '';
            foreach ($config['bdxz'] as $t) {
                if ($t['site'] == $pc_site && !$pc_token) {
                    $pc_token = $t['token'];
                    $pc_appid = $t['appid'];
                }
                if ($t['site'] == $mb_site && !$mb_token) {
                    $mb_token = $t['token'];
                    $mb_appid = $t['appid'];
                }
            }
            if ($pc_appid && $pc_token) {
                $rt = \Phpcmf\Service::M('bdxz', 'bdxz')->bdxz($pc_appid, $pc_token, $pc_urls, 'batch');
                if (!$rt['code']) {
                    $this->_json(0, dr_lang('提交失败：'. $rt['msg']), 0);
                }
            }
            if ($mb_appid && $mb_token) {
                $rt = \Phpcmf\Service::M('bdxz', 'bdxz')->bdxz($mb_appid, $mb_token, $mb_urls, 'batch');
                if (!$rt['code']) {
                    $this->_json(0, dr_lang('提交失败：'. $rt['msg']), 0);
                }
            }
        } else {
            $this->_json(0, dr_lang('没有配置接口数据'));
        }

        exit($this->_json(1, dr_lang('正在更新中（%s/%s）', $page, $tpage), $page + 1));

    }

    public function log_index() {

        $data = $list = [];
        $file = WRITEPATH.'bdxz_log.php';
        if (is_file(WRITEPATH.'bdxz_log.php')) {
            $data = explode(PHP_EOL, str_replace(array(chr(13), chr(10)), PHP_EOL, file_get_contents($file)));
            $data = $data ? array_reverse($data) : [];
            unset($data[0]);
            $page = max(1, (int)\Phpcmf\Service::L('input')->get('page'));
            $limit = ($page - 1) * SYS_ADMIN_PAGESIZE;
            $i = $j = 0;
            foreach ($data as $v) {
                if ($i >= $limit && $j < SYS_ADMIN_PAGESIZE) {
                    $list[] = $v;
                    $j ++;
                }
                $i ++;
            }
        }

        $total = $data ? max(0, count($data) - 1) : 0;

        \Phpcmf\Service::V()->assign(array(
            'list' => $list,
            'total' => $total,
            'mypages'	=> \Phpcmf\Service::L('input')->page(\Phpcmf\Service::L('Router')->url('bdxz/home/log_index'), $total, 'admin')
        ));
        \Phpcmf\Service::V()->display('log.html');
    }

    public function del() {

        @unlink(WRITEPATH.'bdxz_log.php');

        exit($this->_json(1, dr_lang('操作成功')));
    }

    public function add() {

        $mid = dr_safe_filename($_GET['mid']);
        $ids = \Phpcmf\Service::L('input')->get_post_ids();
        if (!$ids) {
            $this->_json(0, dr_lang('所选数据不存在'));
        } elseif (!$mid) {
            $this->_json(0, dr_lang('模块参数不存在'));
        }

        $this->_module_init($mid);

        $data = \Phpcmf\Service::M()->table(SITE_ID.'_'.$mid)->where_in('id', $ids)->getAll();
        if (!$data) {
            $this->_json(0, dr_lang('所选数据为空'));
        }

        $ct = 0;
        foreach ($data as $t) {
            \Phpcmf\Service::M('bdxz', 'bdxz')->module_bdxz($mid, $t['url'], 'add');
            $ct++;
        }

        exit($this->_json(1, dr_lang('共批量%s个URL', $ct)));
    }


}
