<?php namespace Phpcmf\Controllers\Admin;

class Home extends \Phpcmf\Table
{

    private $spider;

    public function __construct() {
        parent::__construct();
        $this->spider = array_flip(\Phpcmf\Service::M('spider', 'spider')->spider);
        \Phpcmf\Service::V()->assign([
            'menu' => \Phpcmf\Service::M('auth')->_admin_menu(
                [
                    '蜘蛛统计' => [APP_DIR.'/'.\Phpcmf\Service::L('Router')->class.'/index', 'fa fa-industry'],
                    '蜘蛛记录' => [APP_DIR.'/'.\Phpcmf\Service::L('Router')->class.'/log_index', 'fa fa-bug'],
                    '域名设置' => [APP_DIR.'/'.\Phpcmf\Service::L('Router')->class.'/edit', 'fa fa-cog'],
                    '统计代码' => [APP_DIR.'/'.\Phpcmf\Service::L('Router')->class.'/code_index', 'fa fa-code'],
                ]
            ),
            'spider' => $this->spider,
        ]);
    }

    public function excel_index() {

        $sq = $sql = dr_safe_replace(\Phpcmf\Service::L('input')->get('sq'));
        $sql = dr_authcode($sql, 'DECODE');
        if (!$sql) {
            $this->_admin_msg(0, 'SQL语句解析失败');
        }

        // 去掉limit语句
        $arr = explode(' LIMIT ', $sql);
        $sql = $arr[0];
        if (!$sql) {
            $this->_admin_msg(0, 'SQL语句解析失败');
        }


        $page = max(1, (int)\Phpcmf\Service::L('input')->get('page'));
        $total = max(1, (int)\Phpcmf\Service::L('input')->get('total'));

        $path = WRITEPATH.'temp/spider-excel-'.$sq.'/';
        if ($page == 1) {
            dr_dir_delete($path);
            dr_mkdirs($path);
        }

        if ($page == 999) {
            if (is_file($path.'1.xlsx')) {
                // code...

                $cname = '蜘蛛记录'.date('Y-m-d-H-i-s').'.zip';
                $zipfile = $path.$cname;
                $rt = \Phpcmf\Service::L('file')->zip($zipfile, $path);
                if ($rt) {
                    $this->_admin_msg(0, $rt);
                }
                set_time_limit(0);
                $handle = fopen($zipfile,"rb");
                if (FALSE === $handle) {
                    $this->_admin_msg(0, dr_lang('文件已经损坏'));
                }

                $filesize = filesize($zipfile);
                header('Content-Type: application/octet-stream');
                header("Accept-Ranges:bytes");
                header("Accept-Length:".$filesize);
                header("Content-Disposition: attachment; filename=".$cname);

                while (!feof($handle)) {
                    $contents = fread($handle, 4096);
                    echo $contents;
                    ob_flush();  //把数据从PHP的缓冲中释放出来
                    flush();      //把被释放出来的数据发送到浏览器
                }

                fclose($handle);
                ob_end_clean();

                exit;
            } else {
                $this->_admin_msg(0, dr_lang('文件丢失'));
            }
        }

        $is_fenye = 0;
        if ($total > 10000 or $page>1) {
            $is_fenye = 1;
        }
        if (!class_exists('ZipArchive')) {
            $is_fenye = 0;
        }

        if ($is_fenye) {
            // 分页输出
            $psize = 10000;
            $a = $psize * ($page - 1);
            $sql.= ' LIMIT '.$a.','.$psize;
        }

        // 查询结果
        $list = \Phpcmf\Service::M()->db->query($sql)->getResultArray();
        if (!$list) {
            // 查询完毕
            if (is_file($path.'1.xlsx')) {
                // code...
                $this->_admin_msg(1, '导出完毕', dr_url(APP_DIR.'/home/excel_index', [
                    'sq' => $sq,
                    'page' => 999,
                ]));
            }
            $this->_admin_msg(0, dr_lang('内容查询为空'));
        }

        $data = [];
        $title = ['时间', '蜘蛛', '来源', '标题', '地址']; // 导出的标题格式
        foreach ($list as $t) {
            $data[] = [
                dr_date($t['inputtime']),
                $this->spider[$t['spider']],
                $t['mobile'] ? '移动端' : '电脑端',
                $t['title'],
                $t['url'],
            ];
        }



        // Create new Spreadsheet object
        $spreadsheet = new \PhpOffice\PhpSpreadsheet\Spreadsheet();
        $sheet = $spreadsheet->getActiveSheet();

        // 方法一，使用 setCellValueByColumnAndRow
        //表头
        //设置单元格内容
        foreach ($title as $key => $value) {
            // 单元格内容写入
            $sheet->setCellValueByColumnAndRow($key + 1, 1, $value);
        }
        $row = 2; // 从第二行开始
        foreach ($data as $item) {
            $column = 1;
            foreach ($item as $value) {
                // 单元格内容写入
                $sheet->setCellValueByColumnAndRow($column, $row, $value);
                $column++;
            }
            $row++;
        }



        if ($is_fenye) {
            // 分页输出
            $file = $path.$page.'.xlsx';
            $writer = \PhpOffice\PhpSpreadsheet\IOFactory::createWriter($spreadsheet, 'Xlsx');
            $writer->save($file);


            $this->_admin_msg(1, '正在导出中（'.$page.'）...', dr_url(APP_DIR.'/home/excel_index', [
                'sq' => $sq,
                'page' => $page+1,
            ]));
        } else {
            $name = '蜘蛛记录'.dr_date(SYS_TIME, 'YmdHis');
            // Redirect output to a client’s web browser (Xlsx)
            header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
            header('Content-Disposition: attachment;filename="'.urlencode($name).'.xlsx"');
            header('Cache-Control: max-age=0');
            // If you're serving to IE 9, then the following may be needed
            header('Cache-Control: max-age=1');

            // If you're serving to IE over SSL, then the following may be needed
            header('Expires: Mon, 26 Jul 1997 05:00:00 GMT'); // Date in the past
            header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT'); // always modified
            header('Cache-Control: cache, must-revalidate'); // HTTP/1.1
            header('Pragma: public'); // HTTP/1.0

            $writer = \PhpOffice\PhpSpreadsheet\IOFactory::createWriter($spreadsheet, 'Xlsx');
            $writer->save('php://output');
            exit;
        }

    }

    public function log_index() {

        $field = [
            'title' => array(
                'ismain' => 1,
                'name' => dr_lang('标题'),
                'fieldname' => 'title',
                'fieldtype' => 'Text',
            ),
            'domain' => array(
                'ismain' => 1,
                'name' => dr_lang('域名'),
                'fieldname' => 'domain',
                'fieldtype' => 'Text',
            ),
            'url' => array(
                'ismain' => 1,
                'name' => dr_lang('地址'),
                'fieldname' => 'url',
                'fieldtype' => 'Text',
            ),
            'ip' => array(
                'ismain' => 1,
                'name' => dr_lang('IP'),
                'fieldname' => 'ip',
                'fieldtype' => 'Text',
            ),
            'agent' => array(
                'ismain' => 1,
                'name' => dr_lang('agent'),
                'fieldname' => 'agent',
                'fieldtype' => 'Text',
            ),
        ];

        $p =[];
        $where = ['site='.SITE_ID];
        $status = \Phpcmf\Service::L('input')->request('status');
        if ($status) {
            foreach ($status as $v) {
                $where[] = 'spider = '.intval($v);
            }
            $p['status'] = $status;
        }

        $this->_init([
            'table' => 'app_spider',
            'field' => $field,
            'where_list' => implode(' AND ', $where),
            'date_field' => 'inputtime',
            'order_by' => 'inputtime desc',
        ]);

        list($tpl) = $this->_List($p);

        \Phpcmf\Service::V()->assign([
            'field' => $field,
        ]);
        \Phpcmf\Service::V()->display('index.html');
    }


    public function index() {

        $tid = intval($_GET['tid']);

        $date_to = dr_safe_replace($_GET['date_to']);
        $date_form = dr_safe_replace($_GET['date_form']);

        if ($date_to && $date_form) {
            $tid = 99;
            $stime = strtotime($date_form.' 00:00:00');
            $etime = strtotime($date_to.' 23:59:59');
        } else {
            switch ($tid) {

                case 2:
                    // 本周
                    $stime=mktime(0,0,0,date('m'),date('d')-date('w')+1,date('y'));
                    $etime=SYS_TIME;
                    break;
                case 1:
                    // 昨天的
                    $stime=mktime(0,0,0,date('m'),date('d')-1,date('Y'));
                    $etime=mktime(0,0,0,date('m'),date('d'),date('Y'))-1;
                    break;

                default:
                    // 今天的
                    $stime = mktime(0,0,0,date('m'),date('d'),date('Y'));
                    $etime = mktime(0,0,0,date('m'),date('d')+1,date('Y'))-1;
                    break;
            }
        }



        $select = [
            0 => '今天',
            1 => '昨天',
            2 => '本周',
        ];

        $table = \Phpcmf\Service::M()->dbprefix('app_spider');

        // 统计蜘蛛
        $zzt = [
            'name' => [],
            'data' => [],
        ];
        $sql = 'SELECT spider,count(*) AS counts from `'.$table.'` where `site`='.SITE_ID.' and `inputtime` BETWEEN '.$stime.' AND '.$etime.' GROUP BY spider; ';
        $rows = \Phpcmf\Service::M()->db->query($sql)->getResultArray();
        if ($rows) {
            foreach ($rows as $t) {
                if ($this->spider[$t['spider']]) {
                    $zzt['name'][] = $this->spider[$t['spider']];
                    $zzt['data'][] = [
                        'value' => $t['counts'],
                        'name' => $this->spider[$t['spider']],
                    ];
                }
            }
        }
        $zzt['name'] = json_encode($zzt['name']);
        $zzt['data'] = json_encode($zzt['data']);

        // 时间段统计
        $sjd = [
            'name' => [
                '00:00',
                '04:00',
                '08:00',
                '12:00',
                '16:00',
                '20:00'
            ],
            'data' => [
                0,0,0,0,0,0
            ]
        ];
        if ($rows) {

            $sql = "SELECT @@GLOBAL.sql_mode;";
            $res = \Phpcmf\Service::M()->db->query($sql);
            if ($res && strpos(dr_array2string($res->getResultArray()), 'ONLY_FULL_GROUP_BY') !== false) {
                $rows = [
                    [
                        'id' => 0,
                        'title' => 'Mysql('.\Phpcmf\Service::M()->db->getVersion().')版本不支持统计',
                        'url' => 'Mysql('.\Phpcmf\Service::M()->db->getVersion().')版本不支持统计',
                    ]
                ];
            } else {

                $sql = 'SELECT
    COUNT(DISTINCT id) counts,
    FROM_UNIXTIME(
        inputtime,
        \'%H\'
    ) AS hours
FROM
    `'.$table.'`
WHERE  `site`='.SITE_ID.' and `inputtime` BETWEEN '.$stime.' AND '.$etime.' 
GROUP BY
    hours
ORDER BY inputtime';
                $rows = \Phpcmf\Service::M()->db->query($sql)->getResultArray();
                if ($rows) {
                    foreach ($rows as $t) {
                        if ($t['hours'] >= 0 && $t['hours'] < 4) {
                            $sjd['data'][0]+= (int)$t['counts'];
                        } elseif ($t['hours'] >= 4 && $t['hours'] < 8) {
                            $sjd['data'][1]+= (int)$t['counts'];
                        } elseif ($t['hours'] >= 8 && $t['hours'] < 12) {
                            $sjd['data'][2]+= (int)$t['counts'];
                        } elseif ($t['hours'] >= 12 && $t['hours'] < 16) {
                            $sjd['data'][3]+= (int)$t['counts'];
                        } elseif ($t['hours'] >= 16 && $t['hours'] < 20) {
                            $sjd['data'][4]+= (int)$t['counts'];
                        } elseif ($t['hours'] >= 20) {
                            $sjd['data'][5]+= (int)$t['counts'];
                        }
                    }
                }

                // 按访问页面分组
                $sql = 'SELECT *,count(*) AS counts from `'.$table.'` where `site`='.SITE_ID.' and  `inputtime` BETWEEN '.$stime.' AND '.$etime.' GROUP BY url  order by counts desc limit 50; ';
                $rows = \Phpcmf\Service::M()->db->query($sql)->getResultArray();
            }
        }

        $sjd['name'] = json_encode($sjd['name']);
        $sjd['data'] = json_encode($sjd['data']);


        \Phpcmf\Service::V()->assign([
            'stime' => $stime,
            'etime' => $etime,
            'zzt' => $zzt,
            'sjd' => $sjd,
            'list' => $rows,
            'tid' => $tid,
            'select' => $select,
            'date_to' => $date_to,
            'date_form' => $date_form,
        ]);
        \Phpcmf\Service::V()->display('fenxi.html');
    }

    public function edit() {

        $domain = [];
        $myfile = WRITEPATH.'config/domain_spider.php';
        if (is_file($myfile)) {
            $domain = require $myfile;
        }

        if (IS_POST) {
            $domain = \Phpcmf\Service::L('input')->post('domain', true);
            if ($domain) {
                $domain = explode(PHP_EOL, $domain);
                file_put_contents(WRITEPATH.'config/domain_spider.php', '<?php return '.var_export($domain, true).';');
            }
            file_put_contents(WRITEPATH.'config/max_spider.php', '<?php return '.intval(\Phpcmf\Service::L('input')->post('max', true)).';');
            $this->_json(1, dr_lang('操作成功'));
        }

        $sys = [];
        $site = require WRITEPATH.'config/domain_site.php';
        foreach ($site as $i => $t) {
            $sys[] = $i;
        }

        $max = \Phpcmf\Service::R(WRITEPATH.'config/max_spider.php') ;

        \Phpcmf\Service::V()->assign([
            'sys' => $sys,
            'domain' => $domain,
            'max' => $max,
        ]);
        \Phpcmf\Service::V()->display('domain.html');
    }

    public function code_index() {
        \Phpcmf\Service::V()->assign([

            'code' => '
{if defined(\'SC_HTML_FILE\')}
<script>
$.ajax({
     type: "get",
     url: "{SITE_URL}index.php?s=spider&url={urlencode($my_web_url)}&title="+document.title,
     dataType: "jsonp"
 });
</script>
{/if}           
            ',
        ]);
        \Phpcmf\Service::V()->display('code.html');
    }

    public function del() {
        $this->_init([
            'table' => 'app_spider',
        ]);
        $this->_Del(
            \Phpcmf\Service::L('Input')->get_post_ids(),
            null,
            null,
            \Phpcmf\Service::M()->dbprefix($this->init['table'])
        );
    }

    public function all_del() {
        \Phpcmf\Service::M()->db->table('app_spider')->truncate();
        $this->_json(1, '清空完毕');
    }

}
