<?php namespace Phpcmf\Model\Spider;

use AlibabaCloud\CDRS\V20201101\PaginateDevice;

class Spider extends \Phpcmf\Model
{

    public $spider = [
        '百度' => 1,
        '谷歌' => 2,
        '搜搜' => 3,
        '必应' => 4,
        '雅虎' => 5,
        '搜狗' => 6,
        '360搜索' => 7,
        '有道' => 8,
        '头条' => 9,
    ];

    // 获取蜘蛛名称
    public function get_spider($string) {

        if (!$string) {
            return '';
        }

        if (stripos($string, 'baiduspider') !== false) {
            return '百度';
        } elseif (stripos($string, 'googlebot') !== false) {
            return '谷歌';
        } elseif (stripos($string, 'sosospider') !== false) {
            return '搜搜';
        } elseif (stripos($string, 'bingbot') !== false) {
            return '必应';
        } elseif (stripos($string, 'Yahoo! Slurp') !== false) {
            return '雅虎';
        } elseif (stripos($string, 'Sogou') !== false) {
            return '搜狗';
        } elseif (stripos($string, '360Spider') !== false) {
            return '360搜索';
        } elseif (stripos($string, 'YoudaoBot') !== false) {
            return '有道';
        } elseif (stripos($string, 'toutao') !== false || stripos($string, 'Bytespider') !== false) {
            return '头条';
        }

        return '';
    }

    // 蜘蛛名称id
    private function _get_spider_id($name) {
        return (int)$this->spider[$name];
    }

    // 域名白名单检测
    private function _domain_check($domain) {

        if (is_file(WRITEPATH.'config/domain_spider.php')) {
            $bmd = require WRITEPATH.'config/domain_spider.php';
            if (in_array($domain, $bmd)) {
                return 1;
            }
        }

        $site = require WRITEPATH.'config/domain_site.php';
        if (isset($site[$domain])) {
            return 1;
        }

        return 0;
    }

    public function run($data) {
        $spider = $this->get_spider($_SERVER['HTTP_USER_AGENT']);
        if (!$spider) {
            return '';
        }
        $url = dr_now_url();
        $rt = $this->save_log($url, $spider, \Phpcmf\Service::C()->_is_mobile(), $data['meta_title'], $url);
        if ($rt && CI_DEBUG) {
            log_message('error', '蜘蛛插件：'.$rt.'（'.$url.'）');
        }
    }

    // 储存日志内容
    public function save_log($url, $spider, $is_mobile, $title) {

        $spider_id = $this->_get_spider_id($spider);
        if (!$spider_id) {
            return '当前访客不是蜘蛛';
        }


        if (!$title) {
            return '网页标题为空';
        }

        $arr = parse_url($url);
        $domain = $arr['host']; // 域名识别
        if (!$domain) {
            return '域名为空';
        }

        if (!$this->_domain_check($domain)) {
            return '域名已被过滤';
        }

        // 过滤无用数据
        if (strpos($url, '?debugbar') !== false
            || strpos($url, 'c=api') !== false
            || strpos($url, 'index.php?s=weixin') !== false
            || strpos($url, 'ajax') !== false
            || strpos($url, 'index.php?s=api&c=module') !== false
            || strpos($url, '&c=html&m=') !== false
            || strpos($url, 'jQuery') !== false
            || strpos($url, '{"code":') !== false
        ) {
            return;
        }

        $max = \Phpcmf\Service::R(WRITEPATH.'config/max_spider.php') ;
        if ($max) {
            $rs = $this->table('app_spider')->counts();
            if ($rs > $max) {
                $this->query('delete from `'.$this->dbprefix('app_spider').'` order by id asc limit 1');
            }
        }

       $this->table('app_spider')->insert([
            'site' => SITE_ID,
            'spider' => $spider_id,
            'domain' => $domain,
            'title' => $title,
            'url' => $url,
            'ip' => \Phpcmf\Service::L('input')->ip_address(),
            'mobile' => $is_mobile ? 1 : 0,
            'agent' => $_SERVER['HTTP_USER_AGENT'],
            'inputtime' => SYS_TIME,
        ]);
        /*
        if ($rt['code'] > 500000) {
            // 超过50w分表存储
            $now = \Phpcmf\Service::M()->dbprefix('app_spider');
            $new = $now.'_'.dr_date(SYS_TIME, 'Y_m_d');
            \Phpcmf\Service::M()->query_all('create table `'.$new.'` select * from `'.$now.'`');
            \Phpcmf\Service::M()->query_all('TRUNCATE `'.$now.'`');
        }*/

        return '';
    }

}